@@ -26,11 +26,12 @@ from tqdm import tqdm
 
 from .io import LatticeData,  save_img, save_img_workflow
 
-from .utils import read_imagej_roi, get_first_last_image_and_task,modify_workflow_task,get_all_py_files, as_type, process_custom_workflow_output
+from .utils import read_imagej_roi, get_first_last_image_and_task,modify_workflow_task,get_all_py_files, as_type, process_custom_workflow_output,check_dimensions
 from . import config
 from napari_workflows import Workflow, WorkflowManager
 from napari_workflows._io_yaml_v1 import load_workflow
 
+
 def _napari_lattice_widget_wrapper():
     #split widget type enables a resizable widget
     @magicclass(widget_type="split")
@@ -43,7 +44,7 @@ def _napari_lattice_widget_wrapper():
             dask = False
             file_name = ""
             save_name = ""
-
+            
             main_heading = widgets.Label(value="<h3>Napari Lattice: Visualization & Analysis</h3>")
             heading1 = widgets.Label(value="Drag and drop an image file onto napari.\nOnce image has opened, initialize the\nplugin by clicking the button below.\nEnsure the image layer and voxel sizes are accurate in the prompt.\n If everything initalises properly, the button turns green.")
             @set_design(background_color="magenta", font_family="Consolas",visible=True,text="Initialize Plugin", max_height=75, font_size = 13) 
@@ -51,7 +52,8 @@ def _napari_lattice_widget_wrapper():
                          pixel_size_dy={"widget_type": "FloatSpinBox", "value":0.1449922, "step": 0.000000001},
                          pixel_size_dz={"widget_type": "FloatSpinBox", "value":0.3, "step": 0.000000001},
                          last_dimension_channel = {"widget_type": "ComboBox","choices":["Channel","Time","Get_from_metadata"],"value":"Get_from_metadata","label":"Set Last dimension (channel/time)","tooltip":"If the last dimension is initialised incorrectly, you can assign it as either channel/time"},
-                         merge_all_channel_layers = {"widget_type":"CheckBox","value":True,"label":"Merge all napari layers as channels","tooltip":"Use this option if the channels are in separate layers. napari-lattice requires all channels to be in same layer"}
+                         merge_all_channel_layers = {"widget_type":"CheckBox","value":True,"label":"Merge all napari layers as channels","tooltip":"Use this option if the channels are in separate layers. napari-lattice requires all channels to be in same layer"},
+                         skew_dir = {"widget_type": "ComboBox","choices":["Y","X"],"value":"Y","label":"Direction of skew (Y or X)","tooltip":"Skew direction when image is acquired. Ask your microscopist for details"}
                          )
             def Choose_Image_Layer(self,
                                       img_layer:Layer,
@@ -67,9 +69,11 @@ def _napari_lattice_widget_wrapper():
                 assert skew_dir in ('Y', 'X'), "Skew direction not recognised. Enter either Y or X"
                 if skew_dir == "X":
                     LLSZWidget.LlszMenu.deskew_func = cle.deskew_x
+                    LLSZWidget.LlszMenu.skew_dir = skew_dir
                 elif skew_dir == "Y":
                     LLSZWidget.LlszMenu.deskew_func = cle.deskew_y
-
+                    LLSZWidget.LlszMenu.skew_dir = skew_dir
+                    
                 if last_dimension_channel == "Get_from_metadata":
                     last_dimension_channel = None
                     
@@ -116,8 +120,7 @@ def _napari_lattice_widget_wrapper():
                 print("Pixel size (ZYX): ",(LLSZWidget.LlszMenu.lattice.dz,LLSZWidget.LlszMenu.lattice.dy,LLSZWidget.LlszMenu.lattice.dx))
                 print("Dimensions of image layer (ZYX): ",list(LLSZWidget.LlszMenu.lattice.data.shape[-3:]))
                 print("Dimensions of deskewed image (ZYX): ",LLSZWidget.LlszMenu.lattice.deskew_vol_shape)
-
-                
+              
                 #Add dimension labels
                 #if channel, and not time
                 if LLSZWidget.LlszMenu.lattice.time == 0 and (last_dimension_channel or LLSZWidget.LlszMenu.lattice.channels>0):
@@ -171,7 +174,7 @@ def _napari_lattice_widget_wrapper():
                 LLSZWidget.LlszMenu.dask = not use_GPU
                 return
 
-            #Redfishlion library for deconvolution
+            #Pycudadecon library for deconvolution
             deconvolution = vfield(bool, name="Use Deconvolution") # options={"enabled": True},
             deconvolution.value = False
             @deconvolution.connect
@@ -186,7 +189,7 @@ def _napari_lattice_widget_wrapper():
             
             @set_design(background_color="magenta", font_family="Consolas",visible=True,text="Click to select PSFs for deconvolution", max_height=75, font_size = 11)
             @set_options(header = dict(widget_type="Label",label="<h3>Enter path to the PSF images</h3>"),
-                         psf_ch1_path={"widget_type": "FileEdit","label":"Channel 1/Multichannel PSF:"},
+                         psf_ch1_path={"widget_type": "FileEdit","label":"Channel 1:"},
                          psf_ch2_path={"widget_type": "FileEdit","label":"Channel 2"},
                          psf_ch3_path={"widget_type": "FileEdit","label":"Channel 3"},
                          psf_ch4_path={"widget_type": "FileEdit","label":"Channel 4"},
@@ -220,10 +223,10 @@ def _napari_lattice_widget_wrapper():
                 
                 
 
-        @magicclass
+        @magicclass(widget_type="collapsible")
         class Preview:          
             @magicgui(header=dict(widget_type="Label",label="<h3>Preview Deskew</h3>"),
-                      time=dict(label="Time:",max= 2**20),
+                      time=dict(label="Time:",max= 2**15),
                       channel=dict(label="Channel:"),
                       call_button="Preview")
             def Preview_Deskew(self, 
@@ -277,7 +280,7 @@ def _napari_lattice_widget_wrapper():
                                save_as_type,
                                save_path)
                    return
-               
+            
             @magicclass(name="Crop and Deskew",widget_type="scrollable")
             class CropWidget:  
                 
@@ -321,7 +324,7 @@ def _napari_lattice_widget_wrapper():
                             # TODO: Add assertion to check if bbox layer or coordinates
                             time = self.time_crop.value
                             channel = self.chan_crop.value
-
+                            
                             assert time < LLSZWidget.LlszMenu.lattice.time, "Time is out of range"
                             assert channel < LLSZWidget.LlszMenu.lattice.channels, "Channel is out of range"
                             
@@ -406,7 +409,7 @@ def _napari_lattice_widget_wrapper():
                             return
                 
                 
-                    @magicclass(name="Crop and Save Data",widget_type="scrollable")
+                    @magicclass(name="Crop and Save Data")
                     class CropSaveData:
                         @magicgui(header=dict(widget_type="Label", label="<h3>Crop and Save Data</h3>"),
                                   time_start=dict(label="Time Start:"),
@@ -430,11 +433,9 @@ def _napari_lattice_widget_wrapper():
                                 print("No coordinates found or cropping. Initialise shapes layer and draw ROIs.")
                             else:
                                 assert LLSZWidget.LlszMenu.open_file, "Image not initialised"
-                                assert 0<= time_start <=LLSZWidget.LlszMenu.lattice.time, "Time start should be >0 or same as total time "+str(CropWidget.CropMenu.lattice.time)
-                                assert 0<=time_end <=LLSZWidget.LlszMenu.lattice.time, "Time end should be between 0 and total time "+str(CropWidget.CropMenu.lattice.time)
-                                assert 0<= ch_start <= LLSZWidget.LlszMenu.lattice.channels, "Channel start should be 0 or >0 or same as no. of channels "+str(CropWidget.CropMenu.lattice.channels)
-                                assert 0<= ch_end <= LLSZWidget.LlszMenu.lattice.channels, "Channel end should be >0 or same as no. of channels " +str(CropWidget.CropMenu.lattice.channels)
-                        
+                                
+                                check_dimensions(time_start,time_end,ch_start,ch_end,LLSZWidget.LlszMenu.lattice.channels,LLSZWidget.LlszMenu.lattice.time)
+                                
                                 angle = LLSZWidget.LlszMenu.lattice.angle
                                 dx = LLSZWidget.LlszMenu.lattice.dx
                                 dy = LLSZWidget.LlszMenu.lattice.dy
@@ -559,8 +560,8 @@ def _napari_lattice_widget_wrapper():
                         print(user_workflow)
                         
                         #when using fields, self.time_preview.value 
-                        assert time_preview <= LLSZWidget.LlszMenu.lattice.time, "Time is out of range"
-                        assert chan_preview <= LLSZWidget.LlszMenu.lattice.channels, "Channel is out of range"
+                        assert time_preview < LLSZWidget.LlszMenu.lattice.time, "Time is out of range"
+                        assert chan_preview < LLSZWidget.LlszMenu.lattice.channels, "Channel is out of range"
 
                         time = time_preview
                         channel = chan_preview
@@ -627,7 +628,8 @@ def _napari_lattice_widget_wrapper():
                                                                 deconvolution=LLSZWidget.LlszMenu.deconvolution.value,
                                                                 decon_processing=LLSZWidget.LlszMenu.lattice.decon_processing,
                                                                 otf_path=otf_path,
-                                                                psf=LLSZWidget.LlszMenu.lattice.psf[channel])
+                                                                psf=LLSZWidget.LlszMenu.lattice.psf[channel],
+                                                                skew_dir=LLSZWidget.LlszMenu.skew_dir)
                                 else:
                                     user_workflow.set("crop_deskew_image",crop_volume_deskew,
                                                                 original_volume = vol_zyx, 
@@ -638,7 +640,8 @@ def _napari_lattice_widget_wrapper():
                                                                 voxel_size_y =LLSZWidget.LlszMenu.lattice.dy, 
                                                                 voxel_size_z =LLSZWidget.LlszMenu.lattice.dz, 
                                                                 z_start = z_start, 
-                                                                z_end = z_end)
+                                                                z_end = z_end,
+                                                                skew_dir=LLSZWidget.LlszMenu.skew_dir)
                                 
                                 #Set input of the workflow to be  crop_deskewing, i.e., the original first operation will now have crop_deskew_image as an input (becoming second instead)
                                 user_workflow.set(input_arg_first,"crop_deskew_image")
@@ -804,11 +807,9 @@ def _napari_lattice_widget_wrapper():
                             save_path (Path, optional): Path to save resulting data
                         """                
                         assert LLSZWidget.LlszMenu.open_file, "Image not initialised"
-                        assert 0<= time_start <=LLSZWidget.LlszMenu.lattice.time, "Time start should be 0 or >0 or same as total time "+str(LLSZWidget.LlszMenu.lattice.time)
-                        assert 0<= time_end <=LLSZWidget.LlszMenu.lattice.time, "Time end should be >0 or same as total time "+str(LLSZWidget.LlszMenu.lattice.time)
-                        assert 0<= ch_start <= LLSZWidget.LlszMenu.lattice.channels, "Channel start should be 0 or >0 or same as no. of channels "+str(LLSZWidget.LlszMenu.lattice.channels)
-                        assert 0<= ch_end <= LLSZWidget.LlszMenu.lattice.channels, "Channel end should be >0 or same as no. of channels " +str(LLSZWidget.LlszMenu.lattice.channels)
-
+                        
+                        check_dimensions(time_start,time_end,ch_start,ch_end,LLSZWidget.LlszMenu.lattice.channels,LLSZWidget.LlszMenu.lattice.time)
+                        
                         #Get parameters
                         angle = LLSZWidget.LlszMenu.lattice.angle
                         dx = LLSZWidget.LlszMenu.lattice.dx
@@ -894,7 +895,8 @@ def _napari_lattice_widget_wrapper():
                                                 z_end = z_end,
                                                 deconvolution=LLSZWidget.LlszMenu.deconvolution.value,
                                                 decon_processing=LLSZWidget.LlszMenu.lattice.decon_processing,
-                                                psf=psf_arg)
+                                                psf=psf_arg,
+                                                skew_dir=LLSZWidget.LlszMenu.skew_dir)
 
                             #change the first task so it accepts "crop_deskew as input"
                             new_task = modify_workflow_task(old_arg=input_arg_first,task_key=task_name_start,new_arg="crop_deskew_image",workflow=user_workflow)
@@ -1062,6 +1064,7 @@ def _napari_lattice_widget_wrapper():
                         
                 pass
     LLSZWidget.WidgetContainer.DeskewWidget.max_width = 100
+    LLSZWidget.WidgetContainer.CropWidget.Preview_Crop_Menu.max_width = 100
     #max_height = 50
     #Important to have this or napari won't recognize the classes and magicclass qidgets
     widget = LLSZWidget()
