@@ -6,13 +6,13 @@ import resource_backed_dask_array
 from typing import Union
 from napari.layers.shapes import shapes
 
-from .utils import calculate_crop_bbox
+from .utils import calculate_crop_bbox, pad_image_nearest_multiple
 
 
 #pass shapes data from single ROI to crop the volume from original data
 def crop_volume_deskew(original_volume:Union[da.core.Array,np.ndarray,cle._tier0._pycl.OCLArray,resource_backed_dask_array.ResourceBackedDaskArray], 
-                        deskewed_volume:Union[da.core.Array,np.ndarray,cle._tier0._pycl.OCLArray,resource_backed_dask_array.ResourceBackedDaskArray], 
-                        roi_shape:Union[shapes.Shapes,list,np.array], 
+                        deskewed_volume:Union[da.core.Array,np.ndarray,cle._tier0._pycl.OCLArray,resource_backed_dask_array.ResourceBackedDaskArray]=None, 
+                        roi_shape:Union[shapes.Shapes,list,np.array]=None, 
                         angle_in_degrees:float=30, 
                         voxel_size_x:float=1, 
                         voxel_size_y:float=1, 
@@ -23,28 +23,35 @@ def crop_volume_deskew(original_volume:Union[da.core.Array,np.ndarray,cle._tier0
                         deconvolution:bool = False,
                         decon_processing:str=None,
                         psf=None,
-                        otf_path=None,
                         num_iter:int=10,
-                        linear_interpolation=True):
-
-    """
-        Uses coordinates from deskewed space to find corresponding coordinates in original volume 
-        and deskew only the specific volume
+                        linear_interpolation=True,
+                        skew_dir:str="Y",
+                        get_deskew_and_decon:bool=False):
+    """Crop the volume from original data and deskew the cropped volume
     Args:
-        original_volume (np.array): Volume to deskew (zyx)
-        deskewed_volume (np.array): Deskewed volume (zyx)
-        roi_shape ((Union[shapes.Shapes,list,np.array])): if shapes layer or data passed, use onl
-        angle_in_degrees ([float): deskewing angle in degrees
-        voxel_size_x (float): [description]
-        voxel_size_y (float): [description]
-        voxel_size_z (float): [description]
-        z_start (int): [description]
-        z_end (int): [description]
-        debug (bool) : False , Can be used to return the uncropped volume for debugging purposes
+        original_volume (Union[da.core.Array,np.ndarray,cle._tier0._pycl.OCLArray,resource_backed_dask_array.ResourceBackedDaskArray]): Volume to deskew (zyx)
+        deskewed_volume:DEPRECATED
+        roi_shape (Union[shapes.Shapes,list,np.array]): shapes layer or rois
+        angle_in_degrees (float, optional): deskewing angle in degrees. Defaults to 30.
+        voxel_size_x (float, optional): microns. Defaults to 1.
+        voxel_size_y (float, optional): microns.  Defaults to 1.
+        voxel_size_z (float, optional): microns.  Defaults to 1.
+        z_start (int, optional): Currently not used, but can be used to crop the volume in z. Defaults to 0.
+        z_end (int, optional): _description_. Currently not used, but can be used to crop the volume in z. Defaults to 1.
+        debug (bool, optional): If True, returns the cropped volume and the cropped volume with extra bounds. Defaults to False.
+        deconvolution (bool, optional): Perform deconvolution. Defaults to False.
+        decon_processing (str, optional): Choose decon option, cuda_gpu or cpu. Defaults to None.
+        psf (_type_, optional): Pass a psf array for deconvolution. Defaults to None.
+        num_iter (int, optional): Number of Iterations for Richardson Lucy deconvolution. Defaults to 10.
+        linear_interpolation (bool, optional): Linear Interpolation after deskewing. Defaults to True.
+        get_deskew_no_decon (bool, optional): Return both deconvolved data and deskewed data with no deconvolution. Defaults to False.
+
+    Returns:
+        _type_: _description_
     """
 
     assert len(original_volume.shape) == 3, print("Shape of original volume must be 3")
-    assert len(deskewed_volume.shape) == 3, print("Shape of deskewed volume must be 3")
+    #assert len(deskewed_volume.shape) == 3, print("Shape of deskewed volume must be 3")
     #assert len(shape) == 4, print("Shape must be an array of shape 4 ")
     shape = None
 
@@ -68,7 +75,7 @@ def crop_volume_deskew(original_volume:Union[da.core.Array,np.ndarray,cle._tier0
     crop_bounding_box, crop_vol_shape = calculate_crop_bbox(shape,z_start,z_end)
     
     #get reverse transform by rotating around original volume
-    reverse_aff, excess_bounds, deskew_transform = get_inverse_affine_transform(original_volume,angle_in_degrees,voxel_size_x,voxel_size_y,voxel_size_z)
+    reverse_aff, excess_bounds, deskew_transform = get_inverse_affine_transform(original_volume,angle_in_degrees,voxel_size_x,voxel_size_y,voxel_size_z,skew_dir)
 
     #apply the transform to get corresponding bounding boxes in original volume
     crop_transform_bbox = np.asarray(list(map(lambda x: reverse_aff._matrix @ x,crop_bounding_box)))
@@ -127,61 +134,95 @@ def crop_volume_deskew(original_volume:Union[da.core.Array,np.ndarray,cle._tier0
     #check if deconvolution is checked
     if deconvolution:
         if decon_processing == "cuda_gpu":
-            crop_volume = pycuda_decon(image = crop_volume, 
+            crop_volume_processed = pycuda_decon(image = crop_volume, 
                                    psf = psf,
                                    dzdata=voxel_size_z,
                                    dxdata=voxel_size_x,
                                    dzpsf=voxel_size_z,
                                    dxpsf=voxel_size_x,
-                                   num_iter=num_iter)
+                                   num_iter=num_iter,
+                                   cropping=True)
         else:
-            crop_volume = skimage_decon(vol_zyx=crop_volume, 
+            crop_volume_processed = skimage_decon(vol_zyx=crop_volume, 
                                     psf=psf, 
                                     num_iter=num_iter,
                                     clip=False, filter_epsilon=0, boundary='nearest')
-      
-    deskewed_prelim = cle.affine_transform(crop_volume, 
+        
+        deskewed_prelim = cle.affine_transform(crop_volume_processed, 
                                            transform =deskew_transform,
                                            auto_size=True,
                                            linear_interpolation=linear_interpolation)
+        if get_deskew_and_decon:
+            deskewed_no_decon = cle.affine_transform(crop_volume, 
+                                           transform =deskew_transform,
+                                           auto_size=True,
+                                           linear_interpolation=linear_interpolation)
+    else:
+
+        deskewed_prelim = cle.affine_transform(crop_volume, 
+                                           transform =deskew_transform,
+                                           auto_size=True,
+                                           linear_interpolation=linear_interpolation)
+
     
     #The height of deskewed_prelim will be larger than specified shape
     # as the coordinates of the ROI are skewed in the original volume
     #IF CLIPPING HAPPENS FOR Y_START or Y_END, use difference to calculate offset
+    if skew_dir=="Y":
+        deskewed_height = deskewed_prelim.shape[1]
+        crop_height = crop_vol_shape[1]
+        
+        #Find "excess" volume on both sides due to deskewing
+        crop_excess = int(round((deskewed_height  - crop_height)/2)) + out_bounds_correction
+        #Crop in Y
+        deskewed_crop = deskewed_prelim[:,crop_excess:crop_height+crop_excess,:]
+    #IF CLIPPING HAPPENS FOR X_START or X_END, use difference to calculate offset
+    elif skew_dir=="X":
+        deskewed_width = deskewed_prelim.shape[2]
+        crop_width = crop_vol_shape[2]
+        #Find "excess" volume on both sides due to deskewing
+        crop_excess = int(round((deskewed_width  - crop_width)/2)) + out_bounds_correction
+        #Crop in Y
+        deskewed_crop = deskewed_prelim[:,:,crop_excess:crop_width+crop_excess]
+        
     
-    deskewed_height = deskewed_prelim.shape[1]
-    crop_height = crop_vol_shape[1]
-    
-    #Find "excess" volume on both sides due to deskewing
-    crop_excess = int(round((deskewed_height  - crop_height)/2)) + out_bounds_correction
-    #Crop in Y
-    deskewed_crop = deskewed_prelim[:,crop_excess:crop_height+crop_excess,:]
-    
-    # For debugging, ,deskewed_prelim will also be returne which is the uncropped volume
+    # For debugging, ,deskewed_prelim will also be returned which is the uncropped volume
     if debug:
         return deskewed_crop,deskewed_prelim
+    elif get_deskew_and_decon:
+        if skew_dir=="Y":
+            deskewed_crop_no_decon = deskewed_no_decon[:,crop_excess:crop_height+crop_excess,:]
+        elif skew_dir=="X":
+            deskewed_crop_no_decon = deskewed_no_decon[:,:,crop_excess:crop_width+crop_excess]
+        return deskewed_crop,deskewed_crop_no_decon
     else:
         return deskewed_crop 
 
 #Get reverse affine transform by rotating around a user-specified volume
-def get_inverse_affine_transform(original_volume,angle_in_degrees,voxel_x,voxel_y,voxel_z):
+def get_inverse_affine_transform(original_volume,
+                                angle_in_degrees,
+                                voxel_x,voxel_y,voxel_z,
+                                skew_dir="Y"):
     """
     Calculate the inverse deskew transform and the excess z_bounds 
-    Difference from using inverse on deskew_y transform is the rotation here is fixed around a
-    specified volume and final affine matrix will be based on the ref volume used
+
     Args:
         original_volume (_type_): _description_
         angle_in_degrees (_type_): _description_
         voxel_x (_type_): _description_
         voxel_y (_type_): _description_
         voxel_z (_type_): _description_
+        skew_dir (str): Direction of skew
 
     Returns:
         Inverse Affine transform (cle.AffineTransform3D), int: Excess z slices, Deskew transform (cle.AffineTransform3D)
     """    
     #calculate the deskew transform for specified volume
-    deskew_transform = _deskew_y_vol_transform(original_volume,angle_in_degrees,voxel_x,voxel_y,voxel_z)
-
+    if skew_dir == "Y":
+        deskew_transform = _deskew_y_vol_transform(original_volume,angle_in_degrees,voxel_x,voxel_y,voxel_z)
+    elif skew_dir == "X":
+        deskew_transform = _deskew_x_vol_transform(original_volume,angle_in_degrees,voxel_x,voxel_y,voxel_z)
+        
     #Get the deskew transform after bringing the volume into bounds
     deskewed_shape, new_deskew_transform, _ = cle._tier8._affine_transform._determine_translation_and_bounding_box(
                                         original_volume, deskew_transform)
@@ -207,7 +248,7 @@ def get_inverse_affine_transform(original_volume,angle_in_degrees,voxel_x,voxel_
 #Get deskew transform where rotation is around centre of "original_volume"
 def _deskew_y_vol_transform(original_volume, angle_in_degrees:float = 30, voxel_size_x: float = 1,
               voxel_size_y: float = 1, voxel_size_z: float = 1, scale_factor: float = 1):
-    """Return deskew transform for specified volume
+    """Return deskew transform for specified volume when skew direction is Y
        Rotation is performed around centre of "original_volume"
     Args:
         crop ([type]): Volume to deskew (zyx)
@@ -241,9 +282,48 @@ def _deskew_y_vol_transform(original_volume, angle_in_degrees:float = 30, voxel_
     transform.rotate(angle_in_degrees= 0 - angle_in_degrees, axis=0)
     # correct orientation so that the new Z-plane goes proximal-distal from the objective.
     
+    return transform
+
+
+#Get deskew transform where rotation is around centre of "original_volume"
+def _deskew_x_vol_transform(original_volume, angle_in_degrees:float = 30, voxel_size_x: float = 1,
+              voxel_size_y: float = 1, voxel_size_z: float = 1, scale_factor: float = 1):
+    """Return deskew transform for specified volume when skew direction is X
+       Rotation is performed around centre of "original_volume"
+    Args:
+        crop ([type]): Volume to deskew (zyx)
+        original_volume ([type]): Reference volume around with to perform rotation (zyx)
+        angle_in_degrees (float): Deskewing angle
+        voxel_size_x (float, optional): [description]. Defaults to 1.
+        voxel_size_y (float, optional): [description]. Defaults to 1.
+        voxel_size_z (float, optional): [description]. Defaults to 1.
+        scale_factor (float, optional): [description]. Defaults to 1.
+
+    Returns:
+        cle.AffineTransform3D
+    """    
+    import math
+    
+    transform = cle.AffineTransform3D()
+    
+    #shear factor for deskewing
+    shear_factor = math.sin((90 - angle_in_degrees) * math.pi / 180.0) * (voxel_size_z/voxel_size_x)
+    transform._matrix[0, 2] = shear_factor
+        
+    # make voxels isotropic, calculate the new scaling factor for Z after shearing
+    # https://github.com/tlamberimage3/napari-ndtiffs/blob/092acbd92bfdbf3ecb1eb9c7fc146411ad9e6aae/napari_ndtiffs/affine.py#L57
+    new_dz = math.sin(angle_in_degrees * math.pi / 180.0) * voxel_size_z
+    scale_factor_z = (new_dz / voxel_size_x) * scale_factor
+    
+    transform.scale(scale_x=scale_factor, scale_y=scale_factor, scale_z=scale_factor_z)
+    
+    #rotation around centre of ref_vol
+    transform.rotate(angle_in_degrees =  angle_in_degrees, axis=1) 
+    # correct orientation so that the new Z-plane goes proximal-distal from the objective.
 
     return transform
 
+
 #deprecated
 #Calculate rotation transform around a volume
 def rotate_around_vol_mat(ref_vol,angle_in_degrees:float=30.0):
@@ -308,10 +388,10 @@ def pycuda_decon(image,otf_path=None,
                  dzpsf=0.3,
                  dxpsf=0.1449922,
                  psf=None,
-                 num_iter:int=10):
+                 num_iter:int=10,
+                 cropping:bool=False):
     """Perform deconvolution using pycudadecon
-    pycudadecon can return cropped images, so we pad the iamges before deconvolution
-    if providing psf, will use that first, if not uses otf_path
+    pycudadecon can return cropped images, so we pad the images with dimensions that are a multiple of 64
 
     Args:
         image (np.array): _description_
@@ -331,22 +411,18 @@ def pycuda_decon(image,otf_path=None,
     if type(image) in [da.core.Array,resource_backed_dask_array.ResourceBackedDaskArray]:
         image = np.array(image)
 
-    import math
     orig_img_shape = image.shape
     
-    #round to nearest 100 as the images get cropped after pycudadecon
-    rounded_shape = tuple([math.ceil(dim/100)*100 for dim in orig_img_shape])
-    #get required padding
-    padding = np.array(rounded_shape) - np.array(orig_img_shape)
-
-    #add padding to image shape
-    padding = np.add(np.array(padding),np.array(orig_img_shape))
-
-    padding_even = [math.ceil(dim // 2) * 2 for dim in padding]
-    padding_even = np.array(padding_even) - np.array(orig_img_shape)
+    #if cropping, add extra padding to avoid edge artefact
+    if cropping:
+        #pad image y dimensionswith half of psf shape
+        z_psf_pad,y_psf_pad,x_psf_pad = np.array(psf.shape) //2
+        image = np.pad(image,((z_psf_pad,z_psf_pad),(y_psf_pad,y_psf_pad),(x_psf_pad,x_psf_pad)),mode="reflect")
+    
+    #pad image to a multiple of 64
+    image = pad_image_nearest_multiple(img = image,nearest_multiple=64)
+    
     
-    image = np.pad(image,((0,padding_even[0]),(0,padding_even[1]),(0,padding_even[2])))
-
     if type(psf) in [np.ndarray,np.array,da.core.Array,resource_backed_dask_array.ResourceBackedDaskArray,cle._tier0._pycl.OCLArray]:
         from pycudadecon import RLContext,TemporaryOTF,rl_decon
         psf = np.squeeze(psf) #remove unit dimensions
@@ -367,7 +443,11 @@ def pycuda_decon(image,otf_path=None,
                 )
         decon_res = rl_decon(image)
         rl_cleanup()
-    #print(decon_res.shape)
+    
+    if cropping:
+        #remove psf padding if cropping was chosen
+        decon_res = decon_res[z_psf_pad:-z_psf_pad,y_psf_pad:-y_psf_pad,x_psf_pad:-x_psf_pad]
+        
     #remove padding; get shape difference and use this shape difference to remove padding
     shape_diff = np.array(decon_res.shape) - np.array(orig_img_shape)
     #if above is negative, 
@@ -377,11 +457,10 @@ def pycuda_decon(image,otf_path=None,
         shape_diff[1] = -orig_img_shape[1]
     if shape_diff[2]==0:
         shape_diff[2] = -orig_img_shape[2]
-        
-    #print(shape_diff)
-    decon_res = decon_res[:-shape_diff[0],:-shape_diff[1],:-shape_diff[2]]
 
-    #make sure image shapes could be different. 
+    decon_res = decon_res[:-shape_diff[0],:-shape_diff[1],:-shape_diff[2]]
+    #print(orig_img_shape)
+    #make sure the decon image and the original image shapes are the same 
     assert decon_res.shape == orig_img_shape, f"Deconvolved {decon_res.shape} and original image shape {orig_img_shape} do not match."
     return decon_res
 
